package test;

import it.poliba.sisinflab.owl.owlapi.MicroReasoner;
import it.poliba.sisinflab.owl.owlapi.MicroReasonerFactory;
import it.poliba.sisinflab.owl.sod.hlds.Abduction;
import it.poliba.sisinflab.owl.sod.hlds.Contraction;

import java.io.File;
import java.util.Iterator;
import java.util.Set;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;

public class TestAbductionContraction {
	
	static String ONTO_DIR = "./dataset/abduction_contraction/toy.owl";
	static String DEMAND_DIR = "./dataset/abduction_contraction/demand_toy.owl";
	static OWLOntology onto = null;
	static OWLOntology onto_demand = null;
	static MicroReasoner reasoner = null;
	
	public static void main(String args[]){
		OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
		File file = new File(ONTO_DIR);
		File demand = new File(DEMAND_DIR);
		
		try{
			onto = manager.loadOntologyFromOntologyDocument(file);
			//Create the factory
			MicroReasonerFactory reasonerFactory = new MicroReasonerFactory();
			//Return an instance of OWLReasoner class that represents our Mini-ME reasoner
			reasoner = reasonerFactory.createMicroReasoner(onto);
			onto_demand = manager.loadOntologyFromOntologyDocument(demand);
			Set<String> demands_name = reasoner.loadDemand(onto_demand);		
			String demandInd = null;
			Iterator<String> iter_demands = demands_name.iterator();
			while(iter_demands.hasNext()){
				demandInd = (String) iter_demands.next();
			}
			
			Set<String> list = reasoner.getSupplyIndividuals(); 
	        Iterator<String> iter = list.iterator();
	        while(iter.hasNext()){
	        	String supplyInd = (String) iter.next();
	        	if(reasoner.checkCompatibility(supplyInd, demandInd) == true){
	        		IRI supplyIRI = IRI.create(supplyInd);
	        		IRI requestIRI = IRI.create(demandInd);
	        		System.out.println("Compute abduction between "+supplyIRI.getFragment()+" e "+requestIRI.getFragment());
	        		Abduction h = reasoner.abduction(supplyInd, demandInd); 
	        		System.out.println("H: "+h.H.toString());
	        	}else {
	        		IRI supplyIRI = IRI.create(supplyInd);
	        		IRI requestIRI = IRI.create(demandInd);
	        		System.out.println("Compute contraction between "+supplyIRI.getFragment()+" e "+requestIRI.getFragment());
	        		Contraction result = reasoner.contraction(supplyInd, demandInd); 
	        		System.out.println("Keep: "+result.K.toString());
	        		System.out.println("Give-up: "+result.G.toString());
	        	}        	
	        }
			
		}catch(Exception e){
			e.printStackTrace();
		}
	}

}
