package test;

import it.poliba.sisinflab.owl.owlapi.MicroReasonerFactory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.reasoner.Node;
import org.semanticweb.owlapi.reasoner.OWLReasoner;

public class TestClassification {
	
	static String ONTO_DIR = "./dataset/classification/building_mod.owl";
	static OWLOntology onto = null;
	static OWLReasoner reasoner = null;
	
	public static void main(String[] args){
		
		try{
			OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
			OWLDataFactory owlDataFactory = OWLManager.getOWLDataFactory();
			File onto_path = new File(ONTO_DIR);
			//Load ontology from an input file
			OWLOntology onto = manager.loadOntologyFromOntologyDocument(onto_path); 
			//Create the factory
			MicroReasonerFactory reasonerFactory = new MicroReasonerFactory();
			//Return an instance of OWLReasoner class that represents our Mini-ME reasoner
			reasoner = reasonerFactory.createMicroReasoner(onto);
			//Print classification result on the standard output.
			System.out.println("Classification output:");
			OWLClass top = owlDataFactory.getOWLThing(); 
            try {
				printClass(top, 0);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			
			
		}catch(OWLOntologyCreationException e){
			e.printStackTrace();
		}	
	}
	
	private static void printClass(OWLClass top, int depth) throws IOException{
        if (!top.getIRI().getFragment().equals("Nothing")){
            String line = "";
            for(int i=0; i<depth; i++){
                line += "    ";
            }
            
            line += top.getIRI().getFragment();
            System.out.println(line);
            
            ArrayList<String> classes = new ArrayList<String>();
            HashMap<String, OWLClass> name = new HashMap<String, OWLClass>();
            Iterator<Node<OWLClass>> it = reasoner.getSubClasses(top, true).iterator();
            while(it.hasNext()){
                OWLClass cls = it.next().getRepresentativeElement();
                String className = cls.getIRI().getFragment();
                if (className != null){
                    classes.add(className);
                    name.put(className, cls);
                }
            }
            Collections.sort(classes);
            
            for(int i=0; i<classes.size(); i++){
                OWLClass subs = name.get(classes.get(i));
                printClass(subs, depth+1);
            }
        }
    }

}
